@@ -22,14 +22,14 @@ get_predict.default <- function(model,
                                 type = "response",
                                 ...) {
 
-    type <- sanity_type(model, type)
+    type <- sanitize_type(model, type)
     type_base <- unname(type)
     type_insight <- names(type)
 
     dots <- list(...)
 
     # some predict methods raise warnings on unused arguments
-    unused <- c("normalize_dydx", "eps", "numDeriv_method", "internal_call", "contrast_numeric_slope")
+    unused <- c("normalize_dydx", "eps", "numDeriv_method", "internal_call", "contrast_numeric_slope", "contrast_numeric", "contrast_factor")
     dots <- dots[setdiff(names(dots), unused)]
 
     # incompatible arguments
